<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
jimport('joomla.client.helper');

class JExtranetControllerFile extends JControllerForm {
	
	protected	$option 		= 'com_jextranet';
	protected 	$text_prefix	= 'COM_JEXTRANET_FILE';
	
	 
	 function upload() {

		JSession::checkToken() or jexit(JText::_('JINVALID_TOKEN'));
		
		// Get the user
		$user		= JFactory::getUser();

		// Get some data from the request
		$file		= JRequest::getVar('Filedata', '', 'files', 'array');
		$folder		= JRequest::getVar('folder', '', '', 'base64');
		$return		= JRequest::getVar('return-url', null, 'post', 'base64');
		

		// Set FTP credentials, if given
		JClientHelper::setCredentialsFromRequest('ftp');

		// Set the redirect
		if ($return) {
			$this->setRedirect(base64_decode($return).'&folder='.$folder);
		}

		// Make the filename safe
		$file['name']	= JFile::makeSafe($file['name']);
		$folder 		= !empty($folder) ? base64_decode( $folder ) : $folder;

		if (isset($file['name'])) {
			
			$base = JExtranetHelper::getSystem()->getUploadFolder();
			
			$filepath = JPath::clean( $base. DIRECTORY_SEPARATOR . $folder . DIRECTORY_SEPARATOR . strtolower($file['name']) );

			$object_file = new JObject($file);
			$object_file->filepath = $filepath;
			$file = (array) $object_file;

			if (JFile::exists($file['filepath'])) {
				// File exists
				JError::raiseWarning(100, JText::_('COM_JEXTRANET_ERROR_FILE_EXISTS'));
				return false;
			} elseif (!$user->authorise('core.create', 'com_jextranet')) {
				// File does not exist and user is not authorised to create
				JError::raiseWarning(403, JText::_('COM_JEXTRANET_ERROR_CREATE_NOT_PERMITTED'));
				return false;
			}

			if (!JFile::upload($file['tmp_name'], $file['filepath'])) {
				// Error in upload
				JError::raiseWarning(100, JText::_('COM_JEXTRANET_ERROR_UNABLE_TO_UPLOAD_FILE'));
				return false;
			} else {
				$this->setMessage(JText::_('COM_JEXTRANET_FILE_UPLOAD_FILE'));
				return true;
			}
			
		} else {
			$this->setRedirect('index.php?option=com_jextranet&view=files', JText::_('COM_JEXTRANET_INVALID_REQUEST'), 'error');
			return false;
		}
	}
	
	
	
	/*
	 * Method to remove files
	 * 
	 * name: 		fdelete 
	 * @param
	 * @return		boolean
	 */
	public function fdelete () {
		
		JRequest::checkToken() or jexit(JText::_('JINVALID_TOKEN'));
		
		$ids = JRequest::getVar('cid', array(), '', 'array');
		$url = 'index.php?option=com_jextranet&view=files';
		
		if (empty($ids)) {
			
			$this->setRedirect( $url, JText::_('COM_JEXTRANET_NO_SELECTED_FILE'), 'warning' );
		 
		 } else {
			 
			$model = $this->getModel();
			
			if ( $model->deleteFiles( $ids )) {
				$this->setRedirect( $url, JText::_('COM_JEXTRANET_FILES_N_ITEMS_DELETED'), 'message' );
			} else {
				$errors = $model->getError();
				$this->setRedirect( $url, $errors,'error' );
			}
		 }
	}
	
	
	
	// we need to define our save function in order to have in field "Parent Folder" selected as default the previous folder used when save and new
	public function save ( $key = null, $urlVar = null ) {
	
		$task 	= $this->getTask(); 
		$folder = JRequest::getVar('folder','','','int');
		$return = JFactory::getApplication()->getUserState('com_jextranet.toReturn', null);
		$url 	= $return ? base64_decode( $return ) : 'index.php?option=com_jextranet&view=folders';
		
		// start redirecting based on action
		if ( $task == 'save2new' ) {
			parent::save();
			$this->setRedirect ( JRoute::_('index.php?option=com_jextranet&view=file&layout=edit&folder=' . $folder, false) );
		} elseif ( $task == 'save' ) {
            parent::save();
			JFactory::getApplication()->setUserState('com_jextranet.toReturn', null );
			$this->setRedirect( $url );
		} else {
			parent::save();
		}
	}
	
	
	// closes the file view and returns to the original view
	public function cancel( $key = null) {
		
		$return = JFactory::getApplication()->getUserState('com_jextranet.toReturn', null);
		$url 	= $return ? base64_decode( $return ) : 'index.php?option=com_jextranet&view=folders';
		JFactory::getApplication()->setUserState('com_jextranet.toReturn', null );
		
		$this->setRedirect( $url );
	}
	
	
	public function edit( $key = NULL, $urlVar = NULL ) {
		$return	= JRequest::getVar('toReturn', null, '', 'base64');
		JFactory::getApplication()->setUserState('com_jextranet.toReturn', $return );
		parent::edit( $key = NULL, $urlVar = NULL );
	}
	

	public function download() {
		
		$model 	= $this->getModel('File', 'JExtranetModel');
		$id  	= JRequest::getVar('id',0,'','int');

		if ( $id ) {
			$file = $model->getFile( $id );
			if ( $file->id ) {
				$model->getHeaders( $file );
				$model->getContent( $file );
			}
		} 
	}
	
}

?>
