<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Copyright (C) 2011 Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */


// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

class JExtranetHelperUser {

	public $juser;
	public $db;
	public $params;


	public function __construct ( $user )
	{
		$this->juser 	= JFactory::getUser($user);
		$this->_db 		= JFactory::getDbo();
		$this->params 	= JComponentHelper::getParams('com_jextranet');
	}



	/*
	 * Method to return the a jextranet user object property
	 */
	public function get( $property ) {
		return $this->juser->get( $property );
	}



	/*
	 * Method to return the jextranet group for user
	 */
	public function getGroup () {

		$query = $this->_db->getQuery( true );
		$query->select('a.g_id, b.g_name, b.g_params');
		$query->from('#__extranet_users AS a');
		$query->join('inner', '#__extranet_users_groups AS b ON a.g_id = b.g_id ');
		$query->where( 'a.u_id = ' . $this->juser->get('id') );

		$this->_db->setQuery($query);
		$group =  $this->_db->loadObject();

		if (empty($group)) {
			$group = new JObject();
			$group->g_name = JText::_('COM_JEXTRANET_USERS_NOGROUP');
			$group->g_id = 0;
		}
		return $group;
	}

	/*
		0 - inherited
		1 - no rights
		2 - list
		3 - download
		4 - upload
		5 - delete
		6 - create folders

	*/
	public function getAuthorisationLevel($folder)
	{
		$user_permissions = $this->getOwnPermissions($folder);
		$group_permissions = $this->getGroupPermissions($folder);

		return $this->clearPermissions(array_unique(array_merge($user_permissions, $group_permissions)));
	}


	public function getOwnPermissions($folder)
	{
		$stored_permissions = $this->getParams()->permissions;
		return $this->clearPermissions($this->getPermissions($folder, $stored_permissions));
	}


	public function getGroupPermissions($folder)
	{
        $p = isset($this->getGroup()->g_params) ?
                json_decode($this->getGroup()->g_params) : new stdclass;
		$stored_permissions = isset($p->permissions) ? $p->permissions : new stdclass;
		return $this->clearPermissions($this->getPermissions($folder, $stored_permissions));
	}


    public function mixPermissions($own, $group)
    {
        $mixed = $this->clearPermissions(
                        array_unique(array_merge($own, $group)));
        sort($mixed);
        return $mixed;
    }

	private function getPermissions($folder, $stored_permissions)
	{
		if ($folder != 'all')
		{
			$f = JExtranetHelper::getFolder($folder);
			$parent = $f->getMainParent();
		}
		else
		{
			$f = new stdclass; $f->type = 'pv'; $f->published = 1; $parent = 0;
		}

		// if folder or main parent folder not published
		if (!empty($f->id) && (!$f->published || !JExtranetHelper::getFolder($parent)->published))
		{
			return array(1);
		}

		// if public folder or the STATUS is set to be public to all folders
		if ($f->type == 'pb' || $this->params->get('folderDefaultStatus') == 2)
		{
			return array(2,3);
		}

		// if he has set permissions to all folders
		if (!empty($stored_permissions->all))
		{
			return is_array($stored_permissions->all) ?
                $stored_permissions->all : array($stored_permissions->all);
		}

		if ($folder == 'all')
		{
			 return array(0);
		}

		// if he has permissions custom
		if ($this->params->get('subfolderPermission') == 1)
		{
			// if he has permissions
			if(isset($stored_permissions->$folder))
			{
				return $stored_permissions->$folder;
			}
			// if not default to the global permissions
			else
			{
				return $this->params->get('globalRights', array(1));
			}
		}

		if ($parent == $folder)
		{
			if(isset($stored_permissions->$folder))
			{
				return $stored_permissions->$folder;
			}
		}
		else
		{
			// if parent is unpublished
			if (!JExtranetHelper::getFolder($parent)->published)
			{
				return array(1);
			}

			// if parent has set permissions
			if(isset($stored_permissions->$parent))
			{
				return $stored_permissions->$parent;
			}
		}

		return $this->params->get('globalRights', array(1));
	}


	public function getVaultFiles()
	{
		$query = $this->_db->getQuery(true);
		$query->select( $this->_db->quoteName('f_id') . ' AS Id,' . $this->_db->quoteName('f_title') . ' AS Title,' .  $this->_db->quoteName('f_type') );
		$query->from( $this->_db->quoteName('#__extranet_users_files') );
		$query->where(  $this->_db->quoteName('f_created_by') . ' = ' . (int) $this->juser->get('id') );
		$query->order('f_id ASC');
		$this->_db->setQuery( $query );
		$files = $this->_db->loadObjectList();
		return $files;
	}

    public function getTable()
    {
        JTable::addIncludePath(JPATH_COMPONENT . '/tables');
        $table = JTable::getInstance('Exuser', 'JExtranetTable');
        $table->load($this->juser->id);
        return $table;
    }

    public function getParams()
    {
        $table = $this->getTable();
        return $table->u_params ? json_decode($table->u_params) : null;
    }

    public function setParams($params)
    {
        $table = $this->getTable();
        $table->u_params = $params;
        $table->store();
    }

	protected function clearPermissions($permissions)
	{
		if (count($permissions)>1 && in_array(1, $permissions))
		{
			$keys = array_keys($permissions, 1);
			foreach ($keys as $k=>$v)
			{
				unset($permissions[$v]);
			}
		}
        sort($permissions);
		return $permissions;
	}
}
