<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');


class JExtranetModelAlert extends JModelAdmin
{

	protected $text_prefix = 'COM_JEXTRANET';
    protected $per_page = 50;


    public function getTable($type = 'Exalert',
                                $prefix = 'JExtranetTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}


	public function getForm($data = array(), $loadData = true)
	{
		// Get the form.
        $form = $this->loadForm('com_jextranet.alert', 'alert',
                        array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)) {
			return false;
		}
		return $form;
	}

	protected function loadFormData()
	{
		// Check the session for previously entered form data.
        $data = JFactory::getApplication()
                    ->getUserState('com_jextranet.edit.alert.data', array());

		if (empty($data)){
			$data = $this->getItem();
		}
		$this->loadParams($data);

		return $data;
	}

	protected function prepareTable($table){

		$filter = \JFilterInput::getInstance();

		if (empty($table->a_id)) 
		{
			$table->a_created = JExtranetHelper::getSystem()->getDateToMySQL();
		}

		$data = JRequest::getVar('jform', array(), '', 'array');

		// set params
		$folders = JRequest::getVar('folders', array(), '', 'array');
		$registry = new JRegistry();
		$registry->loadString($table->params);
		$registry->set('folders',$folders);
		$registry->set('recursive', (int) $data['recursive']);
		$registry->set('subscribers', $filter->clean($data['subscribers'], 'string'));
		$registry->set('frontend', $filter->clean($data['frontend'], 'string'));

		$table->params = (string) $registry;
	}

	/*
	 * Method to delete alerts
	 * 
	 * name: 		deleteAlerts 
	 * @param		array $ids		the id of the alerts to be deleted	
	 * @return		
	 */
	public function deleteAlerts(&$ids){
		$table	= $this->getTable();
		foreach($ids as $id){
			if($table->load($id)){
				if (!$table->delete($id)){
					$this->setError($table->getError());
					return false;
				}
			}
		}
		return true;
	}

    public function getJsonItems($parent, $offset)
    {
        $response = new stdclass;
        $folders = $this->getFolders($parent);
        $response->pages = ceil(count($folders)/$this->per_page);
        $response->items = json_encode(array_slice($folders,
                                $offset*$this->per_page, $this->per_page));
        $response->selection = json_encode($this->getSelectedFolders());
        return $response;
    }

    protected function getSelectedFolders()
    {
        $item = $this->getItem();
        return !empty($item->params['folders']) ? $item->params['folders'] :
                                                array();
    }

    protected function getFolders($parent = 0)
    {
        $query = $this->_db->getQuery(true);
        $query->select('fd_id AS id,fd_name AS name');
        $query->from('#__extranet_folders');
        $query->where('fd_parent = ' . $parent);
        $query->where('fd_type = '. $this->_db->quote('pv'));
        $query->where('published = 1');
        $query->order('ordering DESC');

        $this->_db->setQuery($query);
        return $this->_db->loadObjectList();
    }

    protected function loadParams(&$data) {
    	if (!empty($data->params)) {
    		foreach ($data->params as $key => $value) {
    			if (isset($key)) {
    				$data->{$key} = $value;
    			}
    		}
    	}
    }
}