<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');


jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
jimport('joomla.client.helper');

class JExtranetModelFolder extends JModelAdmin
{

	protected $text_prefix = 'COM_JEXTRANET';


	public function getTable($type = 'Folder', $prefix = 'JExtranetTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}


	public function getForm($data = array(), $loadData = true)
	{
		// Get the form.
		$form = $this->loadForm('com_jextranet.folder', 'folder', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)) {
			return false;
		}
		
		return $form;
	}

	protected function loadFormData()
	{
		// in case we add a subfolder and the parent folder is specified
		$folder = JRequest::getVar('folder','','','int');
		if ( $folder ) {
			$data['fd_parent'] = $folder;
			JFactory::getApplication()->setUserState('com_jextranet.edit.folder.data', $data );
		}

		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_jextranet.edit.folder.data', array());

		if (empty($data)) {
			$data = $this->getItem();
		}

		return $data;
	}


	public function save( $data ) {

		
		// if new folder, set ordering and creation date
		if ( !$data['fd_id'] ) {
			$data['ordering'] 		= $this->getMaxOrder( $data['fd_parent'] ) + 1;
			$data['fd_created'] 	= JExtranetHelper::getSystem()->getDateToMySQL();
		}
	
		// if the folder is moved to a new parent
		if ( $data['fd_id'] && $data['fd_parent'] != $data['fd_parent_orig'] ) {

			
			// FIRST let's check if he did not chosen a subfolder as the new parent, it happens
			$folder 		= JExtranetHelper::getFolder( $data['fd_id'] ); 
			$new_parent		= JExtranetHelper::getFolder( $data['fd_parent'] ); 
			$subfolders 	= $folder->getSubfolders( true ) ;
			$isParent		= $folder->isParentFor( $data['fd_parent'] );
			$current_name 	= JFile::getName( $folder->uploadPath );
			
			// if the folder has subfolders and the new parent is one of the subfolders return false
			if ( $subfolders && $isParent ) {
				return false;
			}
			
			// if we cannot copy the folder to the new location
	
			if ( !JFolder::move( $folder->uploadPath, $new_parent->uploadPath . DIRECTORY_SEPARATOR . $current_name ) ) { 
				return false; 
			}

			
			if ( $subfolders) {
				foreach ( $subfolders as $subfolder ){
					$newpath = str_replace($folder->ftp, $new_parent->ftp . DIRECTORY_SEPARATOR . $current_name,  $subfolder->ftp);
					$this->_updateFoldersPath( $subfolder,  $newpath );
					$this->_updateFilesPath( $subfolder, $newpath );
				}
			}
			
			// update db records
			$newpath 	=	$new_parent->ftp . DIRECTORY_SEPARATOR . $current_name;
			$this->_updateFoldersPath( $folder, $newpath );
			$this->_updateFilesPath( $folder, $newpath );
		}
		parent::save( $data );
		return true;
	}
	
	
	
	// method to retrieve max ordering for the current parent folder
	protected function getMaxOrder( $id ) {
		
		$query = $this->_db->getQuery(true);
		$query->select( 'MAX(a.ordering)' );
		$query->from( $this->_db->quoteName('#__extranet_folders') .' AS a');
		$query->where( $this->_db->quoteName('a.fd_parent') . ' = ' . (int) $id );
		$this->_db->setQuery($query);
		
		return $this->_db->loadResult();
	}
	
	
	
	// creates a directory on server and return the path
	public function makeDir( $data ) {
	
		$parent			= JExtranetHelper::getFolder( $data['fd_parent'] ) ; 
		$folder 		= JFile::makeSafe( $data['fd_name'] );
		$folder 		= trim(str_replace (' ','_',$folder));
		$base			= JExtranetHelper::getSystem()->getUploadFolder();	
		$relpath		= JPath::clean ( $parent->ftp . DIRECTORY_SEPARATOR . $folder ); 
		$path			= $base . $relpath;
				

		if (!JFolder::exists( $path )) {
			JFolder::create ( $path, 0750 );
		} else {
			$i = 1; 
			$path =  $base . $relpath . $i;
			while ( JFolder::exists( $path ) ) {
				$i++;
				$path 	= $base . $relpath . $i; 
			}
			JFolder::create ( $path, 0750 );
		}

		return !empty($i) ? $relpath . $i :  $relpath ;
	}
	
	
	
	protected function _updateFoldersPath( $folder, $newpath ) {
  
		$query = $this->_db->getQuery(true);
		$query->update('#__extranet_folders');
		$query->set( 'fd_name_ftp = ' . $this->_db->quote( $newpath ) );
		$query->where('fd_id = ' . $folder->id );
		$this->_db->setQuery( $query );
		$this->_db->query(); 
	}
	
	
	
	
	/*
	 * Method to update the file path for all files contained by a folder
	 * userful when moving a folder to a different location
	 * 
	 * @param		object $folder		parent folder
	 * 				string $new_path	new folder location	
	 * @return	
	 */
	protected function _updateFilesPath( $folder, $newpath ) {
		
		$query = $this->_db->getQuery( true );
		$query->select('a.f_alias');
		$query->from('#__extranet_files AS a');
		$query->where('a.f_parent = ' . $folder->id);
		$this->_db->setQuery( $query );
		$files = $this->_db->loadObjectList();
		
		if ( $files ) {
			foreach ( $files as $file ) {
				$nou 		= '';
				$paths 		= explode( '/', $file->f_alias );
				$nou 		= $newpath . DIRECTORY_SEPARATOR . end( $paths );
				$query->clear();
				$query->update('#__extranet_files');
				$query->set('f_alias = ' . $this->_db->quote( $nou) );
				$query->where('f_alias = ' . $this->_db->quote( $file->f_alias ) );
				$this->_db->setQuery( $query );
				$this->_db->Query();
			}
		}
	}
	
	
	/*
	 * Method to delete selected folders including subolders and files 
	 * 
	 * name: 		deleteFolders
	 * @param		array $ids		the id of the folders to be deleted	
	 * @return		true // no false since will stop loop
	 */
	public function deleteFolders ( &$ids ) {
		
		JClientHelper::setCredentialsFromRequest('ftp');
		
		$table	= $this->getTable();

		foreach ( $ids as $id ) {
			
			$folder		= JExtranetHelper::getFolder( $id ); 
			$subfolders = array_reverse( $folder->getSubfolders( true ) );

			// remove all the folders from mysql and ftp
			foreach ( $subfolders as $sub ) {

				$subfolder 	= JExtranetHelper::getFolder( $sub->id ); 
				if ( !$subfolder->delete() ) {
					$this->setError ( JText::_('COM_JEXTRANET_FOLDERS_ERROR_NO_FOLDER_FTP_DELETED') . $subfolder->ftp );
					return false;
				}	
			}

			// remove the folder
			if ( !$folder->delete() ) {
				$this->setError ( JText::_('COM_JEXTRANET_FOLDERS_ERROR_NO_FOLDER_FTP_DELETED') . $subfolder->ftp );
				return false;
			}
		}

		return true;
	}
	


	/*
	 * Method to save order for folders
	 * 
	 * @param		array	$pks	an array containing the id of the folders to save
	 * @param		array	$order	an array containing the current order of folders
	 * @return		true 
	 */
	public function saveorder($pks = null, $order = null) {

		$query = $this->_db->getQuery(true);

		foreach ( $pks as $key=>$value ) {
			$query->clear();
			$query->update('#__extranet_folders');
			$query->set('ordering =' . $order[$key]);
			$query->where('fd_id = ' . $value);
			$this->_db->setQuery( $query );
			$this->_db->query();
		}

		return true;
	}
	
	

	public function getFolders( $parent ) {

		$query = $this->_db->getQuery(true);
		$query->select(  $this->_db->quoteName('fd_name') .' AS Title,'.  $this->_db->quoteName('fd_id') .' AS Id,'. $this->_db->quoteName('fd_type') . ',' . $this->_db->quoteName('ordering') );
		$query->from( $this->_db->quoteName('#__extranet_folders') );
		$query->where(  $this->_db->quoteName('fd_parent') . ' = ' . (int) $parent );
		$query->order('ordering ASC');
		$this->_db->setQuery( $query );
		$folders = $this->_db->loadAssocList();
		return $folders;
	}
	
	
	public function getFiles( $parent ) {

		$query = $this->_db->getQuery(true);
		$query->select( $this->_db->quoteName('f_id') . ' AS Id,' . $this->_db->quoteName('f_title') . ' AS Title,' .  $this->_db->quoteName('f_type') . ',' . $this->_db->quoteName('ordering') );
		$query->from( $this->_db->quoteName('#__extranet_files') );
		$query->where(  $this->_db->quoteName('f_parent') . ' = ' . (int) $parent );
		$query->order('ordering ASC');
		$this->_db->setQuery( $query );
		$files = $this->_db->loadObjectList();
		return $files;
	}
	

}
?>
