<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Copyright (C) 2011 Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

class JExtranetHelperAlerter
{
	public  $folder;
	private $alerts = array();
	private $files = array();

	public function __construct($folder = 0) {
		$this->folder = $folder;
		$this->alerts = $this->getAlerts();
		$this->filter();
		$this->users = $this->getUsers();
	}	

	public function setFiles($files = array()) {
		$this->files = $files;
	}

	public function notify() {

		$from = JFactory::getApplication('administrator')->getCfg('mailfrom');
		$fromname = JFactory::getApplication('administrator')->getCfg('fromname');
		$html = (bool) JComponentHelper::getParams('com_jextranet')
					->get('alertFormat', 0);
        $method = JComponentHelper::getParams('com_jextranet')
                    ->get('multipleEmailsMethod', 0);

		foreach ($this->alerts as $alert) {
            $cc = null;
			$bcc = array_map(function($user){return $user->email;}, $this->users);
			if (!empty($alert->params->subscribers)) {
				$bcc = array_merge($bcc, explode(',',$alert->params->subscribers));
			}

			$bcc = array_filter($bcc,function($email){
				return filter_var(trim($email), FILTER_VALIDATE_EMAIL) !== false;});

			$bcc = array_map(function($email){return trim($email);}, $bcc);
			$bcc = array_unique($bcc);

			if (count($bcc) > 0) {

				$to = $bcc[0];
				if (count($bcc) > 1) {
					$bcc = array_slice($bcc, 1);
				} else {
					$bcc = null;
				}
                
                if ($method == 1) {
                    $cc = $bcc;
                    $bcc = null;
                }

				if (!JFactory::getMailer()->sendMail(
					$from, $fromname, $to,
					$this->format($alert->subject), $this->format($alert->message),
					$html, $cc, $bcc)) {
						echo 'error sending front-end emails';
				}
			}
		}
	}

	private function getAlerts() {
		$now = JExtranetHelper::getSystem()->getDateToMySQL();
		$db = JFactory::getDbo();
		$query 	= $db->getQuery( true );
		$regex 	= $db->quote( '"' . (int) $this->folder . '".*"recursive":.*' );
		$parents = $this->parents_search();
		
		$query->select('a.a_id AS id, a.a_description AS message,
			a.a_subject AS subject, a.params');
		$query->from('#__extranet_alerts AS a');
		$query->where( 'a_created <= '. $db->quote( $now ) );
		$query->where('((a.a_publish_up = "0000-00-00 00:00:00") OR 
			(a.a_publish_up <= '. $db->quote( $now ) .'))');
		$query->where('((a.a_publish_down = "0000-00-00 00:00:00") OR 
			(a.a_publish_down > '. $db->quote( $now ) .'))');
		
		if ($parents != '')
		{
			$query->where('(a.params REGEXP ' . $regex . ' OR 
				a.params REGEXP ' . $parents . ')');
		}
		else 
		{
			$query->where('a.params REGEXP ' . $regex);
		}

		$db->setQuery( $query );
		return $db->loadObjectList();
	}

	// filter for front-end activated notifications
	private function filter() {
		$filtered = array();

		foreach ($this->alerts as $key => $alert) {
			$params = json_decode($alert->params);
			if (isset($params->frontend) && $params->frontend == 1) {
				$alert->params = $params;
				$filtered[] = $alert;
			}
		}
		$this->alerts = $filtered;
	}

	private function getUsers() {
		$users = JExtranetHelper::getSystem()->getUsers();
		$usrs = array();

		foreach ($users as $user) {
			$authLevel	= JExtranetHelper::getUser($user->value)
								->getAuthorisationLevel($this->folder);

			if (in_array(2, $authLevel) || in_array(3, $authLevel)) {
				$usrs[]= JFactory::getUser($user->value);
			}
		}

		return $usrs;
	}

	private function parents_search()
	{
		$db = JFactory::getDbo();
		$parents = JExtranetHelper::getFolder($this->folder)->getParent(true);
		return $parents ? 
			$db->quote('("' . implode('"|"',$parents) . '").*"recursive":1.*'):
			'';
	}

	// replace eventually used tags in subject/body
	private function format($text) {
		$website = JFactory::getApplication()->getCfg('sitename');
		$filenames = implode('; ', array_map(function($file){return $file->filename;}, $this->files));
		$folder = JExtranetHelper::getFolder($this->folder);
		$root = JExtranetHelper::getFolder($folder->getMainParent());
		$date = JFactory::getDate('now', JFactory::getApplication()->getCfg('offset'))->format('Y-m-d H:i');
		$user = JFactory::getUser();

		$associations = array(
			'__WEBSITE__' => $website,
			'__FILENAME__' => $filenames,
			'__FOLDER__' => $folder->name,
			'__ROOT__' => $root->name,
			'__DATE__' => $date,
			'__ADDEDBY__' => $user->name
		);

		foreach ($associations as $key => $value) {
			$text = str_replace($key, $value, $text);
		}

		return $text;
	}
}
