<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Copyright (C) 2011 Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

class JExtranetHelperUser {

	public $juser;
	public $db;
	public static $user;
	public $params;
    private static $uparams;
    private static $active;
    private static $group;

	public function __construct ($userId = null) {
		$this->juser 	= JFactory::getUser($userId);
		$this->_db 		= JFactory::getDbo(); 
		$this->params 	= JComponentHelper::getParams('com_jextranet');
	}
	

	public static function getInstance($userId = null) {
		if (!isset(self::$user[$userId])) {
			self::$user[$userId] = new self($userId);
		}
		return self::$user[$userId];
	}


	/*
	 * Method to return the a jextranet user object property
	 */
	public function get( $property ) {
		return $this->juser->get( $property );
	}


	/*
	 * Method to check if the user is also an extranet user
	 */
	public function isExtranetUser() {
        if(!isset(self::$active[$this->juser->id])) {
            $query = $this->_db->getQuery(true);
            $query->select('a.u_id');
            $query->from('#__extranet_users AS a');
            $query->where('a.u_id = ' . $this->juser->get('id'));
            $query->where('a.block = 1');

            $this->_db->setQuery($query);
            $result = $this->_db->loadResult();
            self::$active[$this->juser->id] = (bool) $result;
        }
        return self::$active[$this->juser->id];
	}


	/*
	 * Method to return the jextranet group for user
	 */
	public function getGroup () {
        if (!isset(self::$group[$this->juser->id])) {
            $query = $this->_db->getQuery(true);
            $query->select('a.g_id, b.g_name, b.g_params');
            $query->from('#__extranet_users AS a');
            $query->join('inner',
                '#__extranet_users_groups AS b ON a.g_id = b.g_id ');
            $query->where('a.u_id = ' . $this->juser->get('id'));

            $this->_db->setQuery($query);
            self::$group[$this->juser->id] = $this->_db->loadObject(); 

            if (empty(self::$group[$this->juser->id])) {
                self::$group[$this->juser->id] = new stdclass;
                self::$group[$this->juser->id]->g_name = JText::_('COM_JEXTRANET_USERS_NOGROUP');
                self::$group[$this->juser->id]->g_id = 0;
            }
        }
	    return self::$group[$this->juser->id];
	}
	


	/*
	 * Method to get params for user
	 */
	public function getParams() {
        if(!isset(self::$uparams[$this->juser->id])) {
            $query = $this->_db->getQuery( true );
            $query->select('a.u_params');
            $query->from('#__extranet_users AS a');
            $query->where('a.u_id = ' . $this->juser->get('id') );
            $this->_db->setQuery($query);
            $result =  $this->_db->loadResult();
            self::$uparams[$this->juser->id] = $result ? json_decode($result) : null;
        }
        return self::$uparams[$this->juser->id];
	}


	
	public function getAuthorisationLevel($folder) 
	{
		$user_permissions = $this->getOwnPermissions($folder);
		$group_permissions = $this->getGroupPermissions($folder);

		return $this->clearPermissions(array_unique(array_merge($user_permissions, $group_permissions)));
	}
	

	public function getOwnPermissions($folder)
	{
		$stored_permissions = $this->getParams()->permissions;
		return $this->clearPermissions($this->getPermissions($folder, $stored_permissions));
	}


	public function getGroupPermissions($folder)
	{
		$p = isset($this->getGroup()->g_params) ? json_decode($this->getGroup()->g_params) : new stdclass;
		$stored_permissions = isset($p->permissions) ? $p->permissions : new stdclass;

		return $this->clearPermissions($this->getPermissions($folder, $stored_permissions));
	}

	
	private function getPermissions($folder, $stored_permissions)
	{	
		if ($folder != 'all')
		{
			$f = JExtranetHelper::getFolder($folder);
			$parent = $f->getMainParent();
		}
		else
		{
			$f = new stdclass; $f->type = 'pv'; $f->published = 1; $parent = 0;
		}

		// if folder or main parent folder not published
		if ($f->id && (!$f->published || !JExtranetHelper::getFolder($parent)->published))
		{
			return array(1);
		}

		// if public folder or the STATUS is set to be public to all folders
		if ($f->type == 'pb' || $this->params->get('folderDefaultStatus') == 2)
		{
			return array(2,3);
		}

		// if he has set permissions to all folders
		if (!empty($stored_permissions->all))
		{
			return $stored_permissions->all;
		}

		if ($folder == 'all')
		{
			 return array(0);
		}

		// if he has permissions custom
		if ($this->params->get('subfolderPermission') == 1)
		{	
			// if he has permissions 
			if(isset($stored_permissions->$folder))
			{
				return $stored_permissions->$folder;
			}
			// if not default to the global permissions
			else
			{
				return $this->params->get('globalRights', array(1));
			}
		}
	
		if ($parent == $folder)
		{	
			if(isset($stored_permissions->$folder))
			{
				return $stored_permissions->$folder;
			}
		}
		else
		{
			// if parent has set permissions
			if(isset($stored_permissions->$parent))
			{
				return $stored_permissions->$parent;
			}
		}

		return $this->params->get('globalRights', array(1));
	}
	

	public function canView( $permissions ) {
		return in_array( 2, $permissions );
	}

	public function canDownload( $permissions ) {
		return in_array( 3, $permissions );
	}

	public function canUpload( $permissions ) {
		return in_array( 4, $permissions );
	}

	public function canDelete( $permissions ) {
		return in_array( 5, $permissions );
	}

	public function canCreateFolder( $permissions ) {
		return in_array( 6, $permissions );
	}
	
	
	public function getIP() {

		$ip = null;

		$header_checks = array(
				'HTTP_CLIENT_IP',
				'HTTP_PRAGMA',
				'HTTP_XONNECTION',
				'HTTP_CACHE_INFO',
				'HTTP_XPROXY',
				'HTTP_PROXY',
				'HTTP_PROXY_CONNECTION',
				'HTTP_VIA',
				'HTTP_X_COMING_FROM',
				'HTTP_COMING_FROM',
				'HTTP_X_FORWARDED_FOR',
				'HTTP_X_FORWARDED',
				'HTTP_X_CLUSTER_CLIENT_IP',
				'HTTP_FORWARDED_FOR',
				'HTTP_FORWARDED',
				'ZHTTP_CACHE_CONTROL',
				'REMOTE_ADDR'
				);

		foreach ($header_checks as $key) {
			if (array_key_exists($key, $_SERVER) === true) { 
				foreach (explode(',', $_SERVER[$key]) as $ip) {
					$ip = trim($ip);
				}
			}
		}

		return $ip;
	}


	protected function clearPermissions($permissions)
	{
		if (count($permissions)>1 && in_array(1, $permissions))
		{
			$keys = array_keys($permissions, 1);
			foreach ($keys as $k=>$v)
			{
				unset($permissions[$v]);
			}
		}
		return $permissions;
	}
	
    public function getCustomPages()
    {
        $params = $this->getParams();
        $pages = array();

        if(!empty($params)) {
            for($i=0; $i<4; $i++) {
                $key = 'customPage' . $i . 'Status';
                if(!empty($params->$key)) {
                    $pages[] = $i;
                }
            }
        }
        return $pages;
    }
}
